#!/bin/bash
# ${copyright}
# ------------------------------------------------------------------------------
# # Install Splunk Universal Forwarder with specified configurations
# Run this script as root

# Get SPLUNK URL
if [ -n "$SPLUNK_URL" ]; then
    splunk_url=$SPLUNK_URL
fi

if [ ! $splunk_url ]; then
    printf "\033[31mMissing required arguments - SPLUNK_URL.\033[0m\n"
    exit 1;
fi

# Detect OS / Distribution
KNOWN_DISTRIBUTION="(Debian|Ubuntu|RedHat|CentOS)"
DISTRIBUTION=$(lsb_release -d 2>/dev/null | grep -Eo $KNOWN_DISTRIBUTION)
SPLUNK_UF_VERSION=8.2.3
SPLUNK_UF_BUILD=cd0848707637

if [ -f /etc/debian_version -o "$DISTRIBUTION" == "Debian" -o "$DISTRIBUTION" == "Ubuntu" ]; then
    OS="Debian"
elif [ -f /etc/redhat-release -o "$DISTRIBUTION" == "RedHat" -o "$DISTRIBUTION" == "CentOS" ]; then
    OS="RedHat"
elif [[ -f /etc/os-release && $(grep "^NAME" /etc/os-release | grep -Eo '".*"' | tr -d \") =~ "Amazon Linux" ]]; then
    OS="EC2"
elif [[ -f /etc/os-release && $(grep "^NAME" /etc/os-release | grep -Eo '".*"' | tr -d \") =~ "openSUSE" ]]; then
    OS="OpenSuse"
elif [[ -f /etc/os-release && $(grep "^PRETTY_NAME" /etc/os-release | grep -Eo '".*"' | tr -d \") =~ "SUSE Linux Enterprise" ]]; then
    OS="EnterpriseSuse"
else
    OS="$(uname -s)"
fi

function fail_with_message {
    echo "$@"
    exit 1
}

# Detect root user
if [ $(echo "$UID") = "0" ]; then
    _sudo=''
else
    _sudo='sudo'
    # ask for password & abort script if incorrect:
    sudo -v || fail_with_message 'Will not install Splunk Forwarder. Incorrect administrator password provided.'
fi

function run_install_uf_nix {
    $_sudo mkdir -p "$INSTALL_LOCATION"
    $_sudo wget -O "splunkforwarder-${SPLUNK_UF_VERSION}-${SPLUNK_UF_BUILD}-Linux-x86_64.tgz" "https://download.splunk.com/products/universalforwarder/releases/${SPLUNK_UF_VERSION}/linux/splunkforwarder-${SPLUNK_UF_VERSION}-${SPLUNK_UF_BUILD}-Linux-x86_64.tgz"
    $_sudo tar xvzf "splunkforwarder-${SPLUNK_UF_VERSION}-${SPLUNK_UF_BUILD}-Linux-x86_64.tgz" -C "$INSTALL_LOCATION"
    if [ $? -ne 0 ] ; then
        echo "Failed to install Splunk Universal Forwarder."
        exit 1
    fi
}

# assign splunk_home based on where user wants from the UI
splunk_home="${INSTALL_LOCATION}/splunkforwarder"
set_install=false

function install_uf_debian {
    echo -e "\033[32m\nStep:Installing Splunk Universal Forwarder on Debian...\n\033[0m"

    if [ -n "$CLOUD" ]; then
        #TODO This may conflict with libcurl3 if it's already installed. Needs to be tested more
        $_sudo apt-get install -y curl
    fi

    if [ -d "$splunk_home" ]; then
        existing_uf_version=$(${INSTALL_LOCATION}/splunkforwarder/bin/splunk -version)
        if [[ $existing_uf_version =~ "8.0.1" ]]; then
            echo -e "\033[32m\nSplunk Universal Forwarder 8.0.1 exists, upgrading Universal forwarder to 8.2.3...\n\033[0m"
            set_install=true
        else
            echo -e "\033[32m\nSplunk Universal Forwarder already exists, skipping installation...\n\033[0m"
        fi
    else
        set_install=true
    fi
    if [ "$set_install" = true ];then
        if ! hash wget 2>/dev/null; then
            $_sudo apt-get install -y wget
        fi
        run_install_uf_nix   
    fi 
}

function install_uf_redhat {
    echo -e "\033[32m\nStep:Installing Splunk Universal Forwarder on Redhat...\n\033[0m"

    if [ -d "$splunk_home" ]; then
        existing_uf_version=$(${INSTALL_LOCATION}/splunkforwarder/bin/splunk -version)
        if [[ $existing_uf_version =~ "8.0.1" ]]; then
            echo -e "\033[32m\nSplunk Universal Forwarder 8.0.1 exists, upgrading Universal forwarder to 8.2.3...\n\033[0m"
            set_install=true
        else
            echo -e "\033[32m\nSplunk Universal Forwarder already exists, skipping installation...\n\033[0m"
        fi
    else
        set_install=true
    fi
    if [ "$set_install" = true ];then
        if ! hash wget 2>/dev/null; then
            $_sudo yum install -y wget
        fi
        run_install_uf_nix
    fi
}

function install_uf_suse {
    echo -e "\033[32m\nStep:Installing Splunk Universal Forwarder on SUSE...\n\033[0m"

    if [ -d "$splunk_home" ]; then
        existing_uf_version=$(${INSTALL_LOCATION}/splunkforwarder/bin/splunk -version)
        if [[ $existing_uf_version =~ "8.0.1" ]]; then
            echo -e "\033[32m\nSplunk Universal Forwarder 8.0.1 exists, upgrading Universal forwarder to 8.2.3...\n\033[0m"
            set_install=true
        else
            echo -e "\033[32m\nSplunk Universal Forwarder already exists, skipping installation...\n\033[0m"
        fi
    else
        set_install=true 
    fi
    if [ "$set_install" = true ];then
        if ! hash wget 2>/dev/null; then
            $_sudo zypper install -y wget
        fi
        run_install_uf_nix
    fi
}

function install_uf_darwin {
    if [ -d "$splunk_home" ]; then
        existing_uf_version=$(${INSTALL_LOCATION}/splunkforwarder/bin/splunk -version)
        if [[ $existing_uf_version =~ "8.0.1" ]]; then
            echo -e "\033[32m\nSplunk Universal Forwarder 8.0.1 exists, upgrading Universal forwarder to 8.2.3...\n\033[0m"
            set_install=true
        else
            echo -e "\033[32m\nSplunk Universal Forwarder already exists, skipping installation...\n\033[0m"
        fi
    else
        set_install=true
    fi
    if [ "$set_install" = true ];then
        $_sudo mkdir -p "$INSTALL_LOCATION"
        $_sudo curl -L -o "splunkforwarder-${SPLUNK_UF_VERSION}-${SPLUNK_UF_BUILD}-darwin-64.tgz" "https://download.splunk.com/products/universalforwarder/releases/${SPLUNK_UF_VERSION}/osx/splunkforwarder-${SPLUNK_UF_VERSION}-${SPLUNK_UF_BUILD}-darwin-64.tgz"
        $_sudo tar xvzf "splunkforwarder-${SPLUNK_UF_VERSION}-${SPLUNK_UF_BUILD}-darwin-64.tgz" -C "$INSTALL_LOCATION"
        if [ $? -ne 0 ] ; then
            echo "Failed to install Splunk Universal Forwarder."
            exit 1
        fi
    fi
}

function install_uf_solaris {
    echo -e "\033[32m\nStep:Installing Splunk Universal Forwarder on Solaris x86...\n\033[0m"

    if [ -d "$splunk_home" ]; then
        existing_uf_version=$(${INSTALL_LOCATION}/splunkforwarder/bin/splunk -version)
        if [[ $existing_uf_version =~ "8.0.1" ]]; then
            echo -e "\033[32m\nSplunk Universal Forwarder 8.0.1 exists, upgrading Universal forwarder to 8.2.3...\n\033[0m"
            set_install=true
        else
            echo -e "\033[32m\nSplunk Universal Forwarder already exists, skipping installation...\n\033[0m"
        fi
    else
        set_install=true
    fi
    if [ "$set_install" = true ];then
        $_sudo mkdir -p "$INSTALL_LOCATION"
        $_sudo wget -O "splunkforwarder-${SPLUNK_UF_VERSION}-${SPLUNK_UF_BUILD}-SunOS-x86_64.tar.Z" "https://download.splunk.com/products/universalforwarder/releases/${SPLUNK_UF_VERSION}/solaris/splunkforwarder-${SPLUNK_UF_VERSION}-${SPLUNK_UF_BUILD}-SunOS-x86_64.tar.Z"
        $_sudo gunzip -c "splunkforwarder-${SPLUNK_UF_VERSION}-${SPLUNK_UF_BUILD}-SunOS-x86_64.tar.Z" | (cd "$INSTALL_LOCATION" && $_sudo tar -xvf -)
        if [ $? -ne 0 ] ; then
            echo "Failed to install Splunk Universal Forwarder."
            exit 1
        fi
    fi
}

splunk_app_infra_uf_config="splunk_app_infra_uf_config"

function set_up_forward_server {
    if [ -n "$RECEIVER_PORT" ]; then
        receiver_port=$RECEIVER_PORT
    else
        # Do not create outputs.conf when receiver port not set. For inctance: when running on Splunk Cloud.
        return
    fi
    outputs_conf="$splunk_home/etc/apps/$splunk_app_infra_uf_config/local/outputs.conf"
    $_sudo touch "$outputs_conf"
    $_sudo chmod a+w "$outputs_conf"

    echo -e "[tcpout]" >> "$outputs_conf"
    echo -e "defaultGroup = splunk-app-infra-autolb-group" >> "$outputs_conf"

    echo -e "\n[tcpout:splunk-app-infra-autolb-group]" >> "$outputs_conf"
    echo -e "disabled = false" >> "$outputs_conf"
    echo -e "server = $splunk_url:$receiver_port" >> "$outputs_conf"
}

function set_up_inputs {
    inputs_conf="$splunk_home/etc/apps/$splunk_app_infra_uf_config/local/inputs.conf"

    $_sudo touch "$inputs_conf"
    $_sudo chmod a+w "$inputs_conf"

    # read in log types
    if [ -n "$LOG_SOURCES" ]; then
        log_source=$LOG_SOURCES
    fi
    if [ -n "$log_source" ]; then
        IFS=',' read -a log_sources <<< "$log_source"
    fi

    setResourceIdForCloud

    for l in "${log_sources[@]}"
    do
        source="$(cut -d '%' -f1 <<< "$l")"
        sourcetype="$(cut -d '%' -f2 <<< "$l")"
        echo -e "\n[monitor://$source]" >> "$inputs_conf"
        echo -e "disabled = false" >> "$inputs_conf"
        if [[ $CLOUD = "AWS" ]]; then
            if [ -n "$instance_id" ] && [ -n "$account_id" ]; then
                echo -e "_meta = Cloud::AWS InstanceId::$instance_id AccountID::$account_id" >> "$inputs_conf"
            else
                echo "ERROR: Please configure Cloud Type, AWS instance ID and AWS Account ID manually in inputs.conf under the respective log sources as follows: _meta = Cloud::AWS InstanceId::<instance_id> AccountID::<account_id>"
            fi
        fi
        if [[ $CLOUD = "Azure" ]]; then
            if [ -n "$azure_resource_id" ] && [ -n "$azure_subscription_id" ]; then
                lowercaseresourceid=$(echo $azure_resource_id | tr '[:upper:]' '[:lower:]')
                echo -e "_meta = Cloud::Azure ResourceId::$azure_resource_id LowerCaseResourceId::$lowercaseresourceid SubscriptionId::$azure_subscription_id" >> "$inputs_conf"
            else
                echo "ERROR: Please configure Cloud Type, Azure Resource ID, Lower case Azure Resource ID and Azure Subscription ID manually in inputs.conf  under the respective log sources as follows: _meta = Cloud::Azure ResourceId::<resource_id> LowerCaseResourceId::<resource_id in lower case> SubscriptionId::<subscription_id>"
            fi
        fi
        # Send to _internal index for internal and collectd logs
        [ ! -z $sourcetype ] && echo -e "sourcetype = $sourcetype" >> "$inputs_conf"
        if [[ $sourcetype = "collectd" ]] || [[ $sourcetype = "uf" ]]; then
            echo -e "index = _internal" >> "$inputs_conf"
        fi;
    done
}

function setResourceIdForCloud {
    if [[ $CLOUD = "AWS" ]]; then
        # Please refer to AWS Docs: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html
        # to see how we can get AWS EC2 metadata information.
        run_curl_cmd "curl -fsL --max-time 3 http://169.254.169.254/latest/meta-data/instance-id" "AWS EC2 Instance ID" 3 3
        instance_id=$output

        run_curl_cmd "curl -fsL --max-time 3 http://169.254.169.254/latest/meta-data/identity-credentials/ec2/info/ | grep AccountId | awk '{ print \$3 }' | sed 's/\"//g'" "AWS Account ID" 3 3
		account_id=$output
    fi
    if [[ $CLOUD = "Azure" ]]; then
        azure_resource_id=$(curl -fsL --max-time 3 --retry 3 -H Metadata:true "http://169.254.169.254/metadata/instance/compute/resourceId?api-version=2019-06-01&format=text")
        if [ $? -ne 0 ]; then
            echo "Failed to get Azure VM ID"
        fi

        azure_subscription_id=$(curl -fsL --max-time 3 --retry 3 -H Metadata:true "http://169.254.169.254/metadata/instance/compute/subscriptionId?api-version=2019-06-01&format=text")
        if [ $? -ne 0 ]; then
            echo "Failed to get Azure Subscription ID"
        fi
    fi
    #GCP
}

function run_curl_cmd {
    retry_times=$3
    timeout=$4
    c=0
    while [ $c -le $((retry_times)) ]; do
        output=$(eval $1)
        if [ -n "$output" ]; then
            break
        fi

        if [ ! $c -eq $retry_times ]; then
            echo "Command to get " $2 " failed, sleeping for $timeout seconds and retrying again."
            sleep $timeout
            c=$((c+1))
            timeout=$(( timeout * 2 ))
        else
            echo "Failed to get " $2 "."
            break
        fi
    done
}

# overwrite default inputs.conf, not forwarding any logs
function clean_up_inputs {
    uf_inputs_conf="$splunk_home/etc/apps/SplunkUniversalForwarder/local/inputs.conf"

    if [ -d "$splunk_home/etc/apps/SplunkUniversalForwarder/local" ]; then
        $_sudo mv "$splunk_home/etc/apps/SplunkUniversalForwarder/local" "$splunk_home/etc/apps/SplunkUniversalForwarder/local.$(date +'%Y%m%d-%H%M%S').bak"
    fi

    $_sudo mkdir -p "$splunk_home/etc/apps/SplunkUniversalForwarder/local"

    $_sudo touch "$uf_inputs_conf"
    $_sudo chmod a+w "$uf_inputs_conf"
    # overwrite default and etc/system/default and SplunkUniversalForwarder/default
    echo -e "[monitor://\$SPLUNK_HOME/var/log/splunk]" >> "$uf_inputs_conf"
    echo -e "disabled = true" >> "$uf_inputs_conf"
    echo -e "\n[monitor://\$SPLUNK_HOME/var/log/splunk/metrics.log]" >> "$uf_inputs_conf"
    echo -e "disabled = true" >> "$uf_inputs_conf"
    echo -e "\n[monitor://\$SPLUNK_HOME/var/log/splunk/splunkd.log]" >> "$uf_inputs_conf"
    echo -e "disabled = true" >> "$uf_inputs_conf"
}

function configure {
    echo -e "\033[32m\nStep:Configure Splunk Universal Forwarder...\n\033[0m"
    # backup uf config app if already exists
    if [ -d "$splunk_home/etc/apps/$splunk_app_infra_uf_config" ]; then
        $_sudo tar -zcvf "$splunk_home/etc/apps/$splunk_app_infra_uf_config.$(date +'%Y%m%d-%H%M%S').tar.gz" -C "$splunk_home/etc/apps/" "$splunk_app_infra_uf_config"
        $_sudo rm -rf "$splunk_home/etc/apps/$splunk_app_infra_uf_config"
    fi
    $_sudo mkdir -p "$splunk_home/etc/apps/$splunk_app_infra_uf_config/local"
    # clean up default and exisiting inputs
    clean_up_inputs
    # set up
    set_up_forward_server
    set_up_inputs

    echo -e "\033[32m\nStep:Enabling boot start for Splunk Universal Forwarder...\n\033[0m"
    $_sudo "$splunk_home/bin/splunk" restart --answer-yes --auto-ports --no-prompt --accept-license
    # SPL-161524
    # Enabling is not idempotent pre-7.3.0
    $_sudo "$splunk_home/bin/splunk" disable boot-start
    $_sudo "$splunk_home/bin/splunk" enable boot-start
}


if [ "$OS" = "Debian" ]; then
    install_uf_debian
    configure
elif [ "$OS" = "RedHat" ]; then
    install_uf_redhat
    configure
elif [ "$OS" = "OpenSuse" ] || [ "$OS" = "EnterpriseSuse" ]; then
    install_uf_suse
    configure
elif [ "$OS" = "Darwin" ]; then
    install_uf_darwin
    configure
elif [ "$OS" = "EC2" ]; then
    install_uf_redhat
    configure
elif [ "$OS" = "SunOS" ]; then
    install_uf_solaris
    configure
else
    echo -e "\033[31m\nNot supported operating system: $OS. Nothing is installed.\n\033[0m"
fi
